@@ -39,6 +39,7 @@ from breezy.clean_tree import (
     )
 from breezy.commit import NullCommitReporter
 from breezy.errors import NoSuchFile
+from breezy.osutils import is_inside
 from breezy.rename_map import RenameMap
 from breezy.trace import note, warning
 from breezy.transform import revert
@@ -49,7 +50,7 @@ from debian.deb822 import Deb822
 from .reformatting import FormattingUnpreservable
 
 
-__version__ = (0, 56)
+__version__ = (0, 58)
 version_string = '.'.join(map(str, __version__))
 SUPPORTED_CERTAINTIES = ['certain', 'confident', 'likely', 'possible', None]
 DEFAULT_MINIMUM_CERTAINTY = 'certain'
@@ -119,22 +120,29 @@ class FixerResult(object):
     """Result of a fixer run."""
 
     def __init__(self, description, fixed_lintian_tags=[],
-                 certainty=None):
+                 certainty=None, patch_name=None,
+                 revision_id=None):
         self.description = description
         self.fixed_lintian_tags = fixed_lintian_tags
         self.certainty = certainty
+        self.patch_name = patch_name
+        self.revision_id = revision_id
 
     def __repr__(self):
-        return "%s(%r, fixed_lintian_tags=%r, certainty=%r)" % (
+        return ("%s(%r, fixed_lintian_tags=%r, certainty=%r, patch_name=%r, "
+                "revision_id=%r)") % (
                 self.__class__.__name__,
-                self.description, self.fixed_lintian_tags, self.certainty)
+                self.description, self.fixed_lintian_tags, self.certainty,
+                self.patch_name, self.revision_id)
 
     def __eq__(self, other):
         if type(other) != type(self):
             return False
         return ((self.description == other.description) and
                 (self.fixed_lintian_tags == other.fixed_lintian_tags) and
-                (self.certainty == other.certainty))
+                (self.certainty == other.certainty) and
+                (self.patch_name == other.patch_name) and
+                (self.revision_id == other.revision_id))
 
 
 class Fixer(object):
@@ -150,7 +158,8 @@ class Fixer(object):
 
     def run(self, basedir, package, current_version, compat_release,
             minimum_certainty=None, trust_package=False,
-            allow_reformatting=False, net_access=True, opinionated=False):
+            allow_reformatting=False, net_access=True, opinionated=False,
+            diligence=0):
         """Apply this fixer script.
 
         Args:
@@ -163,6 +172,7 @@ class Fixer(object):
           allow_reformatting: Allow reformatting of files that are being
             changed
           opinionated: Whether to be opinionated
+          diligence: Level of diligence
         Returns:
           A FixerResult object
         """
@@ -174,6 +184,7 @@ def parse_script_fixer_output(text):
     lines = []
     fixed_tags = []
     certainty = None
+    patch_name = None
     for line in text.splitlines():
         # TODO(jelmer): Do this in a slighly less hackish manner
         try:
@@ -185,15 +196,18 @@ def parse_script_fixer_output(text):
                 fixed_tags = value.strip().split(',')
             elif key == 'Certainty':
                 certainty = value.strip()
+            elif key == 'Patch-Name':
+                patch_name = value.strip()
             else:
                 lines.append(line)
     if certainty not in SUPPORTED_CERTAINTIES:
         raise UnsupportedCertainty(certainty)
-    return FixerResult('\n'.join(lines), fixed_tags, certainty)
+    return FixerResult('\n'.join(lines), fixed_tags, certainty, patch_name)
 
 
 def determine_env(package, current_version, compat_release, minimum_certainty,
-                  trust_package, allow_reformatting, net_access, opinionated):
+                  trust_package, allow_reformatting, net_access, opinionated,
+                  diligence):
     env = dict(os.environ.items())
     env['PACKAGE'] = package
     env['CURRENT_VERSION'] = str(current_version)
@@ -203,6 +217,7 @@ def determine_env(package, current_version, compat_release, minimum_certainty,
     env['REFORMATTING'] = ('allow' if allow_reformatting else 'disallow')
     env['NET_ACCESS'] = ('allow' if net_access else 'disallow')
     env['OPINIONATED'] = ('yes' if opinionated else 'no')
+    env['DILIGENCE'] = str(diligence)
     return env
 
 
@@ -224,7 +239,7 @@ class PythonScriptFixer(Fixer):
     def run(self, basedir, package, current_version, compat_release,
             minimum_certainty=DEFAULT_MINIMUM_CERTAINTY,
             trust_package=False, allow_reformatting=False,
-            net_access=True, opinionated=False):
+            net_access=True, opinionated=False, diligence=0):
         env = determine_env(
             package=package,
             current_version=current_version,
@@ -233,7 +248,8 @@ class PythonScriptFixer(Fixer):
             trust_package=trust_package,
             allow_reformatting=allow_reformatting,
             net_access=net_access,
-            opinionated=opinionated)
+            opinionated=opinionated,
+            diligence=diligence)
         try:
             old_env = os.environ
             old_stderr = sys.stderr
@@ -292,7 +308,7 @@ class ScriptFixer(Fixer):
     def run(self, basedir, package, current_version, compat_release,
             minimum_certainty=DEFAULT_MINIMUM_CERTAINTY,
             trust_package=False, allow_reformatting=False,
-            net_access=True, opinionated=False):
+            net_access=True, opinionated=False, diligence=0):
         env = determine_env(
             package=package,
             current_version=current_version,
@@ -301,7 +317,8 @@ class ScriptFixer(Fixer):
             trust_package=trust_package,
             allow_reformatting=allow_reformatting,
             net_access=net_access,
-            opinionated=opinionated)
+            opinionated=opinionated,
+            diligence=diligence)
         with tempfile.SpooledTemporaryFile() as stderr:
             p = subprocess.Popen(self.script_path, cwd=basedir,
                                  stdout=subprocess.PIPE, stderr=stderr,
@@ -474,34 +491,30 @@ def get_committer(tree):
         return config.get('email')
 
 
-def only_changes_last_changelog_block(tree, changelog_path):
+def only_changes_last_changelog_block(tree, changelog_path, changes):
     """Check whether the only change in a tree is to the last changelog entry.
 
     Args:
       tree: Tree to analyze
       changelog_path: Path to the changelog file
+      changes: Changes in the tree
     Returns:
       boolean
     """
     basis_tree = tree.basis_tree()
     with tree.lock_read(), basis_tree.lock_read():
-        changes = tree.iter_changes(basis_tree)
-        try:
-            first_change = next(changes)
-        except StopIteration:
-            return False
-        try:
-            next(changes)
-        except StopIteration:
-            pass
+        for change in changes:
+            try:
+                change_paths = change.path
+            except AttributeError:  # brz < 3.1
+                change_paths = change[1]
+            if change_paths == ('', ''):
+                continue
+            if change_paths != (changelog_path, changelog_path):
+                return False
+            break
         else:
             return False
-        try:
-            change_paths = first_change.path
-        except AttributeError:  # brz < 3.1
-            change_paths = first_change[1]
-        if change_paths != (changelog_path, changelog_path):
-            return False
         new_cl = Changelog(tree.get_file_text(changelog_path))
         old_cl = Changelog(basis_tree.get_file_text(changelog_path))
         if old_cl.distributions != "UNRELEASED":
@@ -654,11 +667,24 @@ def guess_update_changelog(tree, path='', cl=None):
     return True
 
 
+def has_non_debian_changes(changes):
+    for change in changes:
+        try:
+            change_paths = change.path
+        except AttributeError:  # brz < 3.1
+            change_paths = change[1]
+        for path in change_paths:
+            if path and not is_inside('debian', path):
+                return True
+    return False
+
+
 def run_lintian_fixer(local_tree, fixer, committer=None,
                       update_changelog=None, compat_release=None,
                       minimum_certainty=None, trust_package=False,
                       allow_reformatting=False, dirty_tracker=None,
-                      subpath='.', net_access=True, opinionated=None):
+                      subpath='.', net_access=True, opinionated=None,
+                      diligence=0):
     """Run a lintian fixer on a tree.
 
     Args:
@@ -677,6 +703,7 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
       subpath: Path in tree to operate on
       net_access: Whether to allow accessing external services
       opinionated: Whether to be opinionated
+      diligence: Level of diligence
     Returns:
       tuple with set of FixerResult, summary of the changes
     """
@@ -710,7 +737,8 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
             trust_package=trust_package,
             allow_reformatting=allow_reformatting,
             net_access=net_access,
-            opinionated=opinionated)
+            opinionated=opinionated,
+            diligence=diligence)
     except BaseException:
         reset_tree(local_tree, dirty_tracker, subpath)
         raise
@@ -739,23 +767,40 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
         RenameMap.guess_renames(
             basis_tree, local_tree, dry_run=False)
 
-    if len(local_tree.get_parent_ids()) <= 1:
-        changes = local_tree.iter_changes(
-            basis_tree, specific_files=specific_files)
-        try:
-            next(changes)
-        except StopIteration:
-            raise NoChanges("Script didn't make any changes")
+    changes = list(local_tree.iter_changes(
+        basis_tree, specific_files=specific_files,
+        want_unversioned=False, require_versioned=True))
+
+    if len(local_tree.get_parent_ids()) <= 1 and not changes:
+        raise NoChanges("Script didn't make any changes")
 
     if not result.description:
         raise DescriptionMissing()
 
     summary = result.description.splitlines()[0]
+
+    # If there are upstream changes in a non-native package, perhaps
+    # export them to debian/patches
+    if has_non_debian_changes(changes) and current_version.debian_revision:
+        # TODO(jelmer): Apply all patches before generating a diff.
+        reset_tree(local_tree, dirty_tracker, subpath)
+        raise NoChanges("Creating upstream patches not supported yet")
+
+        from .patches import move_upstream_changes_to_patch
+        try:
+            specific_files, patch_name = move_upstream_changes_to_patch(
+                local_tree, subpath, result.patch_name, result.description,
+                dirty_tracker)
+        except FileExistsError as e:
+            raise NoChanges('patch path %s already exists\n' % e.args[0])
+
+        summary = 'Add patch %s: %s' % (patch_name, summary)
+
     if update_changelog is None:
         update_changelog = guess_update_changelog(local_tree, subpath, cl)
 
     if update_changelog and only_changes_last_changelog_block(
-            local_tree, changelog_path):
+            local_tree, changelog_path, changes):
         # If the script only changed the last entry in the changelog,
         # don't update the changelog
         update_changelog = False
@@ -776,10 +821,12 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
     if committer is None:
         committer = get_committer(local_tree)
 
-    local_tree.commit(description, allow_pointless=False,
-                      reporter=NullCommitReporter(),
-                      committer=committer,
-                      specific_files=specific_files)
+    revid = local_tree.commit(
+        description, allow_pointless=False,
+        reporter=NullCommitReporter(),
+        committer=committer,
+        specific_files=specific_files)
+    result.revision_id = revid
     # TODO(jelmer): Run sbuild & verify lintian warning is gone?
     return result, summary
 
@@ -819,7 +866,7 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
                        compat_release=None, minimum_certainty=None,
                        trust_package=False, allow_reformatting=False,
                        use_inotify=None, subpath='.', net_access=True,
-                       opinionated=None):
+                       opinionated=None, diligence=0):
     """Run a set of lintian fixers on a tree.
 
     Args:
@@ -839,6 +886,7 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
       subpath: Subpath in the tree in which the package lives
       net_access: Whether to allow network access
       opinionated: Whether to be opinionated
+      diligence: Level of diligence
     Returns:
       Tuple with two lists:
         1. list of tuples with (lintian-tag, certainty, description) of fixers
@@ -868,7 +916,8 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
                         allow_reformatting=allow_reformatting,
                         dirty_tracker=dirty_tracker,
                         subpath=subpath, net_access=net_access,
-                        opinionated=opinionated)
+                        opinionated=opinionated,
+                        diligence=diligence)
             except FormattingUnpreservable as e:
                 ret.formatting_unpreservable[fixer.name] = e.path
                 if verbose:
