@@ -41,7 +41,9 @@ def matches_wildcard(text, wildcard):
 class Rule(object):
     """A make rule."""
 
-    def __init__(self, target=None, commands=None, prereq_targets=None):
+    def __init__(self, target=None, commands=None, prereq_targets=None,
+                 precomment=None):
+        self.precomment = precomment or []
         self.target = target
         self.components = prereq_targets or []
         if self.components:
@@ -56,8 +58,8 @@ class Rule(object):
             self.lines = None
 
     @classmethod
-    def _from_first_line(cls, firstline):
-        self = cls()
+    def _from_first_line(cls, firstline, precomment=None):
+        self = cls(precomment=precomment)
         self.lines = [firstline]
         # TODO(jelmer): What if there are multiple targets?
         self.target, self._component_str = firstline.split(b':', 1)
@@ -91,7 +93,7 @@ class Rule(object):
         self.lines.append(line)
 
     def append_command(self, command):
-        self.lines.append(b'\t' + command + b'\n')
+        self.lines.append(b'\t' + command)
 
     def append_component(self, component):
         self.components.append(component)
@@ -99,26 +101,29 @@ class Rule(object):
         self.lines[0] = b'%s:%s' % (self.target, self._component_str)
 
     def dump_lines(self):
-        return [line + b'\n' for line in self.lines]
+        for line in self.precomment:
+            yield line + b'\n'
+        for line in self.lines:
+            yield line + b'\n'
 
     def __eq__(self, other):
         if not isinstance(other, type(self)):
             return False
-        return self.dump_lines() == other.dump_lines()
+        return list(self.dump_lines()) == list(other.dump_lines())
 
     def __bool__(self):
         return bool(self.lines)
 
     def clear(self):
+        self.precomment = []
         self.lines = []
 
-    def _trim_trailing_whitespace(self):
-        while self.lines and not self.lines[-1].strip():
-            del self.lines[-1]
-
-    def _ensure_trailing_whitespace(self):
-        if self.lines and self.lines[-1].strip():
-            self.lines.append(b'')
+    def _finish(self):
+        rest = [self]
+        while self.lines and (
+                not self.lines[-1] or self.lines[-1].startswith(b'#')):
+            rest.insert(1, self.lines.pop(-1))
+        return rest
 
 
 def is_conditional(line):
@@ -143,12 +148,15 @@ class Makefile(object):
             original_contents = f.read()
         return cls.from_bytes(original_contents)
 
-    def iter_rules(self, target, exact=True):
+    def iter_all_rules(self):
         for entry in self.contents:
-            if isinstance(entry, Rule) and entry.has_target(target, exact):
+            if isinstance(entry, Rule):
                 yield entry
-        else:
-            return
+
+    def iter_rules(self, target, exact=True):
+        for rule in self.iter_all_rules():
+            if rule.has_target(target, exact):
+                yield rule
 
     @classmethod
     def from_bytes(cls, contents):
@@ -178,8 +186,13 @@ class Makefile(object):
                     mf.contents.append(line)
             elif b':' in line and b' ' not in line.split(b':')[0]:
                 if rule:
-                    mf.contents.append(rule)
-                rule = Rule._from_first_line(line)
+                    mf.contents.extend(rule._finish())
+                precomment = []
+                while (len(mf.contents) > 1 and
+                        isinstance(mf.contents[-1], bytes) and
+                        mf.contents[-1].startswith(b'#')):
+                    precomment.insert(0, mf.contents.pop(-1))
+                rule = Rule._from_first_line(line, precomment=precomment)
             elif not line.strip():
                 if rule:
                     rule.append_line(line)
@@ -187,20 +200,27 @@ class Makefile(object):
                     mf.contents.append(line)
             else:
                 if rule:
-                    mf.contents.append(rule)
+                    mf.contents.extend(rule._finish())
                 rule = None
                 mf.contents.append(line)
 
         if rule:
-            mf.contents.append(rule)
+            mf.contents.extend(rule._finish())
 
         return mf
 
     def dump_lines(self):
         lines = []
-        for entry in self.contents:
+        contents = self.contents[:]
+        while contents and contents[-1] == b'':
+            del contents[-1]
+        for entry in contents:
             if isinstance(entry, Rule):
-                lines.extend(entry.dump_lines())
+                if entry.lines == []:
+                    if lines[-1] == b'\n':
+                        lines.pop(-1)
+                else:
+                    lines.extend(entry.dump_lines())
             else:
                 lines.append(entry + b'\n')
         return lines
@@ -208,18 +228,16 @@ class Makefile(object):
     def dump(self):
         return b''.join(self.dump_lines())
 
-    def add_rule(self, target, components=None):
+    def add_rule(self, target, components=None, precomment=None):
         if self.contents:
             if isinstance(self.contents[-1], Rule):
-                self.contents[-1]._ensure_trailing_whitespace()
-            elif self.contents[-1].strip():
-                self.contents.append(b'\n')
+                self.contents.append(b'')
         if isinstance(target, list):
             target = b' '.join(target)
         line = b'%s:' % target
         if components:
             line += b' ' + b' '.join(components)
-        rule = Rule._from_first_line(line)
+        rule = Rule._from_first_line(line, precomment=precomment)
         self.contents.append(rule)
         return rule
 
@@ -240,29 +258,24 @@ class MakefileUpdater(Updater):
         return self._parsed
 
 
-class RulesUpdater(Updater):
+class RulesUpdater(MakefileUpdater):
 
     def __init__(self, path='debian/rules'):
         super(RulesUpdater, self).__init__(path)
 
+    def legacy_update(self, command_line_cb=None, global_line_cb=None,
+                      rule_cb=None, makefile_cb=None):
+        """Update a debian/rules file.
 
-def update_makefile(path, command_line_cb=None, global_line_cb=None,
-                    rule_cb=None, makefile_cb=None):
-    """Update a makefile.
-
-    Args:
-      path: Path to the makefile to edit
-      command_line_cb: Callback to call on every rule command line
-      global_line_cb: Callback to call on every global line
-      rule_cb: Callback called for every rule
-    Returns:
-      boolean indicating whether any changes were made
-    """
-    if not os.path.exists(path):
-        return False
-    with MakefileUpdater(path) as updater:
+        Args:
+          command_line_cb: Callback to call on every rule command line
+          global_line_cb: Callback to call on every global line
+          rule_cb: Callback to call on every rule
+        Returns:
+          boolean indicating whether any changes were made
+        """
         newcontents = []
-        for entry in updater.makefile.contents:
+        for entry in self.makefile.contents:
             if isinstance(entry, Rule):
                 rule = entry
                 newlines = []
@@ -290,6 +303,9 @@ def update_makefile(path, command_line_cb=None, global_line_cb=None,
                     rule_cb(rule)
                 if rule:
                     newcontents.append(rule)
+                else:
+                    if newcontents and newcontents[-1] == b'':
+                        newcontents.pop(-1)
             else:
                 line = entry
                 if global_line_cb:
@@ -303,14 +319,15 @@ def update_makefile(path, command_line_cb=None, global_line_cb=None,
                 else:
                     raise TypeError(line)
 
-        if newcontents and isinstance(newcontents[-1], Rule):
-            newcontents[-1]._trim_trailing_whitespace()
-
-        updater.makefile.contents = newcontents
+        self.makefile.contents = newcontents
         if makefile_cb:
-            makefile_cb(updater.makefile)
-
-    return updater.changed
+            makefile_cb(self.makefile)
+        if self.has_changed():
+            for rule in self.makefile.iter_all_rules():
+                discard_pointless_override(rule)
+            return True
+        else:
+            return False
 
 
 def discard_pointless_override(rule):
@@ -337,14 +354,15 @@ def update_rules(command_line_cb=None, global_line_cb=None,
     Returns:
       boolean indicating whether any changes were made
     """
-    changed = update_makefile(
-        path, command_line_cb=command_line_cb, global_line_cb=global_line_cb,
-        rule_cb=rule_cb, makefile_cb=makefile_cb)
-    if changed:
-        update_makefile(path, rule_cb=discard_pointless_override)
-        return True
-    else:
+    if not os.path.exists(path):
         return False
+    with RulesUpdater(path) as updater:
+        updater.legacy_update(
+            command_line_cb=command_line_cb,
+            global_line_cb=global_line_cb,
+            rule_cb=rule_cb,
+            makefile_cb=makefile_cb)
+    return updater.changed
 
 
 def dh_invoke_add_with(line, with_argument):
